{******************************************************************************
*File name: Spromeps.pas

*(C) Copyright 1996-2002 Rainbow Technologies, Inc.  All rights reserved.

* Description: SuperPro Multiple Entry Points Module
    This module provides an alternate method for performing SuperPro API
    commands.  It provides a function for each API command, so you do not
    have to deal with command packets.
******************************************************************************}

UNIT Spromeps;

// This string of statements will define the MSWINDOWS directive on Delphi 5, Delphi 6
// already has this defined. Win32 is not recommended to be used in future versions of
// Delphi because there may be a Win64 define sometime in the future.

{$IFDEF VER130}        // Delphi 5 compiler version number
{$IFDEF WIN32}         // Win32 system (Delphi 5 only supports Win32)
{$DEFINE MSWINDOWS}    // Define MSWINDOWS directive like Delphi 6.
{$ENDIF}
{$ENDIF}

INTERFACE

CONST
 { SuperPro API error codes }
 SP_SUCCESS                     = 0;
 SP_INVALID_FUNCTION_CODE       = 1;
 SP_INVALID_PACKET              = 2;
 SP_UNIT_NOT_FOUND              = 3;
 SP_ACCESS_DENIED               = 4;
 SP_INVALID_MEMORY_ADDRESS      = 5;
 SP_INVALID_ACCESS_CODE         = 6;
 SP_PORT_IS_BUSY                = 7;
 SP_WRITE_NOT_READY             = 8;
 SP_NO_PORT_FOUND               = 9;
 SP_ALREADY_ZERO                = 10;
 SP_DRIVER_OPEN_ERROR           = 11;
 SP_DRIVER_NOT_INSTALLED        = 12;
 SP_IO_COMMUNICATIONS_ERROR     = 13;
 SP_PACKET_TOO_SMALL            = 15;
 SP_INVALID_PARAMETER           = 16;
 SP_MEM_ACCESS_ERROR            = 17;
 SP_VERSION_NOT_SUPPORTED       = 18;
 SP_OS_NOT_SUPPORTED            = 19;
 SP_QUERY_TOO_LONG              = 20;
 SP_INVALID_COMMAND             = 21;
 SP_MEM_ALIGNMENT_ERROR         = 29;
 SP_DRIVER_IS_BUSY              = 30;
 SP_PORT_ALLOCATION_FAILURE     = 31;
 SP_PORT_RELEASE_FAILURE        = 32;
 SP_ACQUIRE_PORT_TIMEOUT        = 39;
 SP_SIGNAL_NOT_SUPPORTED        = 42;
 SP_UNKNOWN_MACHINE             = 44;
 SP_SYS_API_ERROR               = 45;
 SP_UNIT_IS_BUSY                = 46;
 SP_INVALID_PORT_TYPE           = 47;
 SP_INVALID_MACH_TYPE           = 48;
 SP_INVALID_IRQ_MASK            = 49;
 SP_INVALID_CONT_METHOD         = 50;
 SP_INVALID_PORT_FLAGS          = 51;
 SP_INVALID_LOG_PORT_CFG        = 52;
 SP_INVALID_OS_TYPE             = 53;
 SP_INVALID_LOG_PORT_NUM        = 54;
 SP_INVALID_ROUTER_FLGS         = 56;
 SP_INIT_NOT_CALLED             = 57;
 SP_DRVR_TYPE_NOT_SUPPORTED     = 58;
 SP_FAIL_ON_DRIVER_COMM         = 59;

 SP_SERVER_PROBABLY_NOT_UP      = 60;
 SP_UNKNOWN_HOST                = 61;
 SP_SENDTO_FAILED               = 62;
 SP_SOCKET_CREATION_FAILED      = 63;
 SP_NORESOURCES                 = 64;
 SP_BROADCAST_NOT_SUPPORTED     = 65;
 SP_BAD_SERVER_MESSAGE          = 66;
 SP_NO_SERVER_RUNNING           = 67;
 SP_NO_NETWORK                  = 68;
 SP_NO_SERVER_RESPONSE          = 69;
 SP_NO_LICENSE_AVAILABLE        = 70;
 SP_INVALID_LICENSE             = 71;
 SP_INVALID_OPERATION           = 72;
 SP_BUFFER_TOO_SMALL            = 73;
 SP_INTERNAL_ERROR              = 74;
 SP_PACKET_ALREADY_INITIALIZED  = 75;
 SP_PROTOCOL_NOT_INSTALLED      = 76;


 // String for OS Driver type

RB_WINNT_SYS_DRVR        = 5;          // Windows NT system driver
RB_WIN95_SYS_DRVR        = 7;          // Windows 95 system driver
RB_NW_LOCAL_DRVR         = 8;          // Netware local driver

 // EnumFlags Const
 NSPRO_RET_ON_FIRST             = 1;
 NSPRO_GET_ALL_SERVERS          = 2;
 NSPRO_RET_ON_FIRST_AVAILABLE   = 4;

// Protocol flags const
NSPRO_TCP_PROTOCOL             = 1;
NSPRO_IPX_PROTOCOL             = 2;
NSPRO_NETBEUI_PROTOCOL         = 4;
NSPRO_SAP_PROTOCOL             = 8;

// HeartBeat const
MAX_HEARTBEAT                  = 2592000;
MIN_HEARTBEAT                  = 60;
INFINITE_HEARTBEAT             = $FFFFFFFF;

        RNBO_STANDALONE       = 'RNBO_STANDALONE';
	RNBO_SPN_LOCAL        = 'RNBO_SPN_LOCAL';
	RNBO_SPN_DRIVER       = 'RNBO_SPN_DRIVER';
	RNBO_SPN_BROADCAST    = 'RNBO_SPN_BROADCAST';
	RNBO_SPN_ALL_MODES    = 'RNBO_SPN_ALL_MODES';
	RNBO_SPN_SERVER_MODES = 'RNBO_SPN_SERVER_MODES';

// SOME CONSTANT FOR INPUTS VALIDATION
MAX_DEVID_LENGTH        = 4;
MAX_ADDRESS_LENGTH      = 2;
MAX_ACCESS_CODE         = 3;
MIN_ACCESS_CODE         = 0;
MAX_DATA_LENGTH         = 4;
MAX_NUM_SERVER          = 10;
MIN_NUM_SERVER          = 1;
MIN_NUM_LICENESE        = 0;
MAX_NUM_LICENESE        =65535 ;
MAX_PASSWORDS_LENGTH    = 4;

MAX_QUERY_LENGTH        = 56;
MIN_QUERY_LENGTH        = 8;
MAX_SERVER_LENGTH       = 64;




 { const for API packet size }
SPRO_APIPACKET_ALIGNMENT_VALUE = (SizeOf(Longint));
RB_SPRO_APIPACKET_SIZE         = (1024+SPRO_APIPACKET_ALIGNMENT_VALUE);

MAX_NAME_LEN                   = 64;
MAX_ADDR_LEN                   = 32;

 {structure for KeyInfo API}
    type
        NSPRO_KEY_MONITOR_INFO = record
            devId      :WORD;
            hardLimit  :WORD;
            inUse      :WORD;
            numTimeOut :WORD;
            highestUse :WORD;
        end;

{Struct for monitoring info }
    type
         NSPRO_MONITOR_INFO = record
            serverName : ARRAY [0..MAX_NAME_LEN-1] OF CHAR;
            serverIPAddress : ARRAY [0..MAX_ADDR_LEN-1] OF CHAR;
            serverIPXAddress : ARRAY [0..MAX_ADDR_LEN-1] OF CHAR;
            version : ARRAY [0..MAX_NAME_LEN-1] OF CHAR;
            protocol : WORD;
            sproKeyMonitorInfo :   NSPRO_KEY_MONITOR_INFO;
    end;

{struct for  EnumSerevr }
     type
        NSPRO_SERVER_INFO = record
           serverAddress :  ARRAY [0..MAX_ADDR_LEN-1] OF CHAR;
           numLicAvail :WORD;
     end;

TYPE
    RB_SPRO_APIPACKET       = ARRAY [0..RB_SPRO_APIPACKET_SIZE-1] OF CHAR;  { Spro API Packet }
    RB_SPRO_APIPACKET_PTR   = ^RB_SPRO_APIPACKET;
    RB_SPRO_SERVER_NAME     = ARRAY [0..MAX_NAME_LEN-1] OF CHAR;
    NSPRO_MONITOR_INFO_PTR  = ^NSPRO_MONITOR_INFO;
    NSPRO_SERVER_INFO_PTR   = ^NSPRO_SERVER_INFO;

  var
  thePacket : RB_SPRO_APIPACKET;

FUNCTION SproFormatPacket( ApiPacket : RB_SPRO_APIPACKET_PTR ;
                           PacketSize : WORD  ) : WORD;

FUNCTION SproGetKeyInfo( ApiPacket : RB_SPRO_APIPACKET_PTR ;
                                        devID : WORD ;
                                        KeyIndex : WORD;
                                        MonitorInfo : NSPRO_MONITOR_INFO_PTR ) : WORD;

FUNCTION SproEnumServer(enumFlag : Integer;
                        devID : WORD ;
                        serverInfo :     NSPRO_SERVER_INFO_PTR;
                        numServer : POINTER) : WORD;

FUNCTION SproInitialize( ApiPacket : RB_SPRO_APIPACKET_PTR   ) : WORD;

FUNCTION SproGetSubLicense( ApiPacket :  RB_SPRO_APIPACKET_PTR;
                            Address   :  WORD ) : WORD;

FUNCTION SproReleaseLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                 Address : WORD;
                                 NumSubLic : POINTER) : WORD;

FUNCTION SproSetContactServer( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                               ServerName  : string ) : WORD;

FUNCTION SproGetContactServer( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                               ServerName  : PChar;
                               ServerNameLen : WORD) : WORD;

FUNCTION SproGetFullStatus( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD;

FUNCTION SproGetVersion( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                         VAR majVer     : BYTE;
                         VAR minVer     : BYTE;
                         VAR rev        : BYTE;
                         VAR osDrvrType : BYTE ): WORD;

FUNCTION SproFindFirstUnit( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                            developerID : WORD ) : WORD;

FUNCTION SproFindNextUnit( ApiPacket   : RB_SPRO_APIPACKET_PTR ) : WORD;

FUNCTION SproRead ( ApiPacket : RB_SPRO_APIPACKET_PTR;
                    address   : WORD;
                    data      : POINTER ) : WORD;

FUNCTION SproExtendedRead( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                           address        : WORD;
                           data           : POINTER;
                           accessCode     : POINTER) : WORD;

FUNCTION SproWrite( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                    writePassword : WORD;
                    address       : WORD;
                    data          : WORD;
                    accessCode    : WORD ) : WORD;

FUNCTION SproOverwrite ( ApiPacket          : RB_SPRO_APIPACKET_PTR;
                         writePassword      : WORD;
                         overwritePassword1 : WORD;
                         overwritePassword2 : WORD;
                         address            : WORD;
                         data               : WORD;
                         accessCode         : WORD ) : WORD;

FUNCTION SproDecrement( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                        writePassword : WORD;
                        address       : WORD ) : WORD;

FUNCTION SproActivate( ApiPacket         : RB_SPRO_APIPACKET_PTR;
                       writePassword     : WORD;
                       activatePassword1 : WORD;
                       activatePassword2 : WORD;
                       address           : WORD ) : WORD;

FUNCTION SproQuery( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                    address        : WORD;
                    queryData      : POINTER;
                    response       : POINTER;
                    VAR response32 : LONGINT;
                    length         : WORD ) : WORD;

FUNCTION SproGetHardLimit(ApiPacket : RB_SPRO_APIPACKET_PTR;
                          HardLmt   : POINTER ) : WORD;

FUNCTION SproSetHeartBeat(ApiPacket : RB_SPRO_APIPACKET_PTR;
                           heartbeat : LongInt) : WORD;

FUNCTION SproSetProtocol( ApiPacket : RB_SPRO_APIPACKET_PTR;
                           ProtocolFlag : WORD) : WORD;



IMPLEMENTATION


{ External functions }

{$IFDEF MSWINDOWS}  // Function prototypes for the Windows DLL.
FUNCTION RNBOsproFormatPacket( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                               thePacketSize : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproInitialize( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproFindFirstUnit( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                                devleoperID : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproFindNextUnit( ApiPacket    : RB_SPRO_APIPACKET_PTR ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproRead( ApiPacket : RB_SPRO_APIPACKET_PTR;
                       address   : WORD;
                       data      : POINTER ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproExtendedRead( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                               address    : WORD;
                               data       : POINTER;
                               accessCode : POINTER ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproWrite( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                        writePassword : WORD;
                        address       : WORD;
                        data          : WORD;
                        accessCode    : BYTE ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproOverwrite( ApiPacket          : RB_SPRO_APIPACKET_PTR;
                            writePassword      : WORD;
                            overwritePassword1 : WORD;
                            overwritePassword2 : WORD;
                            address            : WORD;
                            data               : WORD;
                            accessCode         : BYTE ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproDecrement( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                            writePassword : WORD;
                            address       : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproActivate( ApiPacket         : RB_SPRO_APIPACKET_PTR;
                           writePassword     : WORD;
                           activatePassword1 : WORD;
                           activatePassword2 : WORD;
                           address           : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproQuery( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                        address    : WORD;
                        queryData  : POINTER;
                        response   : POINTER;
                        response32 : POINTER;
                        length     : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproGetFullStatus( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';
FUNCTION RNBOsproGetVersion( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                             majVer     : POINTER;
                             minVer     : POINTER;
                             rev        : POINTER;
                             osDrvrType : POINTER ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproSetContactServer( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                   ServerName : AnsiString ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproGetContactServer( ApiPacket : RB_SPRO_APIPACKET_PTR;
                              ServerNameBuf : PChar;
                              ServerNameBufSz : WORD) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproGetSubLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                Address : WORD ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproReleaseLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                 Address : WORD;
                                 NumSubLic : POINTER) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproGetHardLimit( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                               HardLmt    : POINTER)  : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproSetHeartBeat(ApiPacket : RB_SPRO_APIPACKET_PTR;
                              heartbeat : LongInt) : WORD;    STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproSetProtocol( ApiPacket : RB_SPRO_APIPACKET_PTR;
                              ProtocolFlag : WORD) : WORD;    STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproGetKeyInfo(ApiPacket : RB_SPRO_APIPACKET_PTR;
                            devId : WORD;
                            keyIndex :WORD;
                            MonitorInfo : NSPRO_MONITOR_INFO_PTR ) : WORD; STDCALL; EXTERNAL 'sx32w.dll';

FUNCTION RNBOsproEnumServer(enumFlag : Integer;
                        devID : WORD ;
                        serverInfo :     NSPRO_SERVER_INFO_PTR;
                        numServer : POINTER) : WORD;STDCALL; EXTERNAL 'sx32w.dll';
{$ENDIF}  {MSWINDOWS}

{$IFDEF LINUX}  // Function prototypes for Linux.
FUNCTION RNBOsproFormatPacket( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                               thePacketSize : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproInitialize( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproFindFirstUnit( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                                devleoperID : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproFindNextUnit( ApiPacket    : RB_SPRO_APIPACKET_PTR ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproRead( ApiPacket : RB_SPRO_APIPACKET_PTR;
                       address   : WORD;
                       data      : POINTER ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproExtendedRead( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                               address    : WORD;
                               data       : POINTER;
                               accessCode : POINTER ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproWrite( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                        writePassword : WORD;
                        address       : WORD;
                        data          : WORD;
                        accessCode    : BYTE ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproOverwrite( ApiPacket          : RB_SPRO_APIPACKET_PTR;
                            writePassword      : WORD;
                            overwritePassword1 : WORD;
                            overwritePassword2 : WORD;
                            address            : WORD;
                            data               : WORD;
                            accessCode         : BYTE ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproDecrement( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                            writePassword : WORD;
                            address       : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproActivate( ApiPacket         : RB_SPRO_APIPACKET_PTR;
                           writePassword     : WORD;
                           activatePassword1 : WORD;
                           activatePassword2 : WORD;
                           address           : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproQuery( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                        address    : WORD;
                        queryData  : POINTER;
                        response   : POINTER;
                        response32 : POINTER;
                        length     : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproGetFullStatus( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD; CDECL; EXTERNAL 'libnspro.so';
FUNCTION RNBOsproGetVersion( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                             majVer     : POINTER;
                             minVer     : POINTER;
                             rev        : POINTER;
                             osDrvrType : POINTER ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproSetContactServer( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                   ServerName : AnsiString ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproGetContactServer( ApiPacket : RB_SPRO_APIPACKET_PTR;
                              ServerNameBuf : PChar;
                              ServerNameBufSz : WORD) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproGetSubLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                Address : WORD ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproReleaseLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                 Address : WORD;
                                 NumSubLic : POINTER) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproGetHardLimit( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                               HardLmt    : POINTER)  : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproSetHeartBeat(ApiPacket : RB_SPRO_APIPACKET_PTR;
                              heartbeat : LongInt) : WORD;    CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproSetProtocol( ApiPacket : RB_SPRO_APIPACKET_PTR;
                              ProtocolFlag : WORD) : WORD;    CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproGetKeyInfo( ApiPacket : RB_SPRO_APIPACKET_PTR;
                             devId : WORD;
                             keyIndex :WORD;
                             MonitorInfo : NSPRO_MONITOR_INFO_PTR ) : WORD; CDECL; EXTERNAL 'libnspro.so';

FUNCTION RNBOsproEnumServer(enumFlag : Integer;
                        devID : WORD ;
                        serverInfo :     NSPRO_SERVER_INFO_PTR;
                        numServer : POINTER) : WORD;CDECL; EXTERNAL 'libnspro.so';
{$ENDIF}  {LINUX}




{******************************************************************************
Procedure       : SproFormatPacket

Purpose         : To Format the API Packet

Inputs          : SuperPro API Packet

Outputs         : Updated API Packet
                  status
******************************************************************************}

FUNCTION SproFormatPacket( ApiPacket : RB_SPRO_APIPACKET_PTR ; PacketSize : WORD ) : WORD;

BEGIN
     SproFormatPacket:= RNBOsproFormatPacket (ApiPacket,PacketSize );

END; { SproInitialize }


{******************************************************************************
Procedure       : SproFormatPacket

Purpose         : To Format the API Packet

Inputs          : SuperPro API Packet

Outputs         : Updated API Packet
                  status
******************************************************************************}

FUNCTION SproGetKeyInfo( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                        devid : WORD ;
                                        KeyIndex : WORD;
                                        MonitorInfo :     NSPRO_MONITOR_INFO_PTR ) : WORD;

BEGIN
     SproGetKeyInfo:= RNBOsproGetKeyInfo (ApiPacket, devid,keyindex,MonitorInfo);

END; { SproInitialize }





FUNCTION SproEnumServer(enumFlag : Integer;
                        devID : WORD ;
                        serverInfo :     NSPRO_SERVER_INFO_PTR;
                        numServer : POINTER) : WORD;


BEGIN
     SproEnumServer:= RNBOsproEnumServer (enumFlag,devid,serverInfo,numServer);

END; { SproInitialize }





{******************************************************************************
Procedure       : SproInitialize

Purpose         : To initialize the SuperPro Driver

Inputs          : SuperPro API Packet

Outputs         : Updated API Packet
                  status
******************************************************************************}

FUNCTION SproInitialize( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD;

{VAR
returnVal : WORD;}

BEGIN
    {returnVal := RNBOsproFormatPacket (ApiPacket, RB_SPRO_APIPACKET_SIZE);

    IF returnVal <> SP_SUCCESS THEN
       SproInitialize := returnVal
    ELSE   }
       SproInitialize := RNBOsproInitialize (ApiPacket);

END; { SproInitialize }


{******************************************************************************
Procedure       : SproGetExtendedStatus

Purpose         : Return the Status (Word) from the previous SuperPro API Call.

Inputs          : SuperPro API Packet

Outputs         : Updated API Packet
                  status
******************************************************************************}

FUNCTION SproGetFullStatus( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD;

BEGIN

    SproGetFullStatus := RNBOsproGetFullStatus (ApiPacket);

END;


{******************************************************************************
Procedure       : SproGetVersion

Purpose         : Returns the driver's version number.

Inputs          : SuperPro API Packet

Outputs         : Updated API Packet
                  majVer     - major version.
                  minVer     - minor version.
                  rev        - revision.
                  osDrvrType - os driver type.
                  status
******************************************************************************}

FUNCTION SproGetVersion( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                         VAR majVer     : BYTE;
                         VAR minVer     : BYTE;
                         VAR rev        : BYTE;
                         VAR osDrvrType : BYTE ): WORD;
VAR
   mav, miv, rv, ot : BYTE;

BEGIN

    SproGetVersion := RNBOsproGetVersion (ApiPacket, @mav, @miv, @rv, @ot);
    majVer         := mav;
    minVer         := miv;
    rev            := rv;
    osDrvrType     := ot;

END;

{******************************************************************************
Procedure       : SproSetContactServer

Purpose         : Set the contact server. In case of using the local server,
                  it can be set to no-net

Inputs          : SuperPro API Packet , ServerName

Outputs         : status
******************************************************************************}

FUNCTION SproSetContactServer( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                               ServerName  : string ) : WORD;
BEGIN

    SproSetContactServer := RNBOsproSetContactServer(ApiPacket, ServerName);

END; { SproSetContactServer }

{?????????? Some problem in Getting}

FUNCTION SproGetContactServer( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                               ServerName  : PChar;
                               ServerNameLen : WORD) : WORD;
BEGIN

    SproGetContactServer := RNBOsproGetContactServer(ApiPacket, ServerName, ServerNameLen);

END; { SproGetContactServer }

{!!!! Chencking Get sub license }
FUNCTION SproGetSubLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                Address : WORD ) : WORD;
BEGIN

    SproGetSubLicense := RNBOsproGetSubLicense(ApiPacket, Address);

END;  { SproGetSubLicense }

{ SproReleaseLicense }
FUNCTION SproReleaseLicense( ApiPacket : RB_SPRO_APIPACKET_PTR;
                                 Address : WORD;
                                 NumSubLic : POINTER) : WORD;
BEGIN
    SproReleaseLicense := RNBOsproReleaseLicense(ApiPacket, Address, NumSubLic);
END; { SproReleaseLicense }

{******************************************************************************
Procedure        : SproGetHardLimit

Purupose         : To get the hard limit of the key

Inputs           : SuperPro API Packet.

Outputs           : Hardlimit
                   Status
******************************************************************************}
FUNCTION SproGetHardLimit( ApiPacket  : RB_SPRO_APIPACKET_PTR;
                           HardLmt    : POINTER)  : WORD;

BEGIN

     SproGetHardLimit := RNBOsproGetHardLimit(ApiPacket,HardLmt);

END;{SproGetHardLimit}


{******************************************************************************
Procedure       : SproFindFirstUnit

Purpose         : Find a SuperPro with the specified developer ID.

Inputs          : SuperPro API Packet.
                  Your assigned 16 bit developer ID.

Outputs         : Updated API Packet.
                  Status
******************************************************************************}

FUNCTION SproFindFirstUnit( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                            developerID : WORD ) : WORD;

BEGIN

    SproFindFirstUnit := RNBOsproFindFirstUnit(ApiPacket, developerID);

END; { SproFindFirstUnit }


{******************************************************************************
Procedure       : SproFindNextUnit

Purpose         : To find the next SuperPro unit with the same developer ID.

Inputs          : SuperPro API Packet.

Outputs         : Updated API packet.
                  Status.
******************************************************************************}

FUNCTION SproFindNextUnit( ApiPacket : RB_SPRO_APIPACKET_PTR ) : WORD;

BEGIN

    SproFindNextUnit := RNBOsproFindNextUnit(ApiPacket);

END; { SproFindNextUnit }


{******************************************************************************
Procedure       : SproRead

Purpose         : To read the data in a readable SuperPro Cell

Inputs          : SuperPro API Packet
                  Address (0 - 63) of the SuperPro word to read

Outputs         : Update SuperPro API Packet
                  SuperPro word data
                  Status
******************************************************************************}

FUNCTION SproRead ( ApiPacket   : RB_SPRO_APIPACKET_PTR;
                    address     : WORD;
                    data        : POINTER ) : WORD;

BEGIN

    SproRead := RNBOsproRead(ApiPacket, address, data);

END; { SproRead }


{******************************************************************************
Procedure       : SproExtendedRead

Purpose         : To read the data in a readable SuperPro Cell

Inputs          : SuperPro API Packet
                  Address (0 - 63) of the SuperPro word to read

Outputs         : Upated SuperPro API Packet
                  SuperPro word data
                  Access Code
                  Status
******************************************************************************}

FUNCTION SproExtendedRead( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                           address        : WORD;
                           data           : POINTER;
                           accessCode     : POINTER) : WORD;

BEGIN

     SproExtendedRead := RNBOsproExtendedRead (ApiPacket,
                                               address,
                                               data, accessCode);

END; { SproExtendedRead }


{******************************************************************************
Procedure       : SproWrite

Purpose         : To write to a read/write Superpro Cell

Inputs          : SuperPro API Packet
                  Write password
                  Address (0 - 63) of the SuperPro word to write
                  Cell data (Word)
                  Access Code

Outputs         : Updated SuperPro API Packet
                  Status

Note            : The access codes are
                    0 - Data Word - read/write
                    1 - Locked Data Word - Read only
                    2 - Counter
                    3 - Algorithm or Password

                  Note: The write command can only write over SuperPro Data
                  words.  To write over any other word requires using
                  SproOverwrite.
******************************************************************************}

FUNCTION SproWrite( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                    writePassword : WORD;
                    address       : WORD;
                    data          : WORD;
                    accessCode    : WORD ) : WORD;

BEGIN

  SproWrite := RNBOsproWrite(ApiPacket, writePassword, address, data, accessCode);

END; { SproWrite }


{******************************************************************************
Procedure       : SproOverwrite

Purpose         : To overwrite any Superpro Word (words 5 - 63)

Inputs          : SuperPro API Packet
                  Write password
                  Overwrite password word 1
                  Overwrite password word 2
                  Address (0 - 63) of the SuperPro word to write
                  Data to write
                  Access code to write (0-3)

Outputs         : Updated SuperPro API Packet
                  status

Description     : The access codes are
                    0 - Data Word - read/write
                    1 - Locked Data Word - Read only
                    2 - Counter
                    3 - Algorithm or Password
******************************************************************************}

FUNCTION SproOverwrite ( ApiPacket          : RB_SPRO_APIPACKET_PTR;
                         writePassword      : WORD;
                         overwritePassword1 : WORD;
                         overwritePassword2 : WORD;
                         address            : WORD;
                         data               : WORD;
                         accessCode         : WORD ) : WORD;

BEGIN

  SproOverWrite :=  RNBOsproOverWrite( ApiPacket,
                                       writePassword,
                                       overwritePassword1,
                                       overwritePassword2,
                                       address,
                                       data,
                                       accessCode );

END; { SproOverwrite }


{******************************************************************************
Procedure       : SproDecrement

Purpose         : To decrement a counter word.

Inputs          : SuperPro API Packet
                  Write password
                  Address (0 - 63) of the SuperPro word to decrement

Outputs         : Updated SuperPro API Packet
                  Status

Description     : Note: If the counter is associated with an active algorithm
                  and the counter is decremented to 0, the associated
                  algorithm will be deactivated.

                  The counter and algorithm can appear in the SuperPro as
                    Address    Data
                     N - 2     Counter
                     N - 1     Counter
                       N       Algorithm word 1
                     N + 1     Algorithm word 1

                  If either or both counters exist, the counter is associated
                  with the algorithm.  This association will only exist for
                  N = 0C, 14, 1C, 24, 2C, 34, 3C Hex.
******************************************************************************}

FUNCTION SproDecrement( ApiPacket     : RB_SPRO_APIPACKET_PTR;
                        writePassword : WORD;
                        address       : WORD) : WORD;

BEGIN

   SproDecrement := RNBOsproDecrement(ApiPacket, writePassword, address);

END; { SproDecrement }


{******************************************************************************
Procedure       : SproActivate

Purpose         : To activate an inactive algo descriptor

Inputs          : SuperPro API Packet
                  Write password
                  Activate password word 1
                  Activate password word 2
                  Address (0 - 63) of the first word of the algorithm to
                    activate

Outputs         : Updated SuperPro API Packet
                  status

Description     : The algorithm and password will appear in the SuperPro as
                    Address    Data
                       N       Algorithm word 1
                     N + 1     Algorithm word 1
                     N + 2     Activate password word 1
                     N + 3     Activate password word 2
******************************************************************************}

FUNCTION SproActivate( ApiPacket         : RB_SPRO_APIPACKET_PTR;
                       writePassword     : WORD;
                       activatePassword1 : WORD;
                       activatePassword2 : WORD;
                       address           : WORD ) : WORD;

BEGIN

   SproActivate :=  RNBOsproActivate( ApiPacket,
                                      writePassword,
                                      activatePassword1,
                                      activatePassword2,
                                      address );

END; { SproActivate }


{******************************************************************************
Procedure       : SproQuery

Purpose         : To scramble a bit stream through the superpro

Inputs          : SuperPro API Packet
                  Address (0 - 63) of the first word of the algorithm to
                    query
                  Pointer to the input string
                  Length of the input (and response) string in bytes

Outputs         : Updated SuperPro API Packet
                  Pointer to the response string
                  Response32 - last 4 bytes of response data
                  Status

Description     : Note: The response string will always be the same length as
                  the query string.

                  Note: If the query and response strings are specified as
                  PASCAL strings, then use @string [1] as the pointer i.e.
                  skip the length byte.  The length byte will need to be set
                  manually in the program.
******************************************************************************}

FUNCTION SproQuery( ApiPacket      : RB_SPRO_APIPACKET_PTR;
                    address        : WORD;
                    queryData      : POINTER;
                    response       : POINTER;
                    VAR response32 : LONGINT;
                    length         : WORD ) : WORD;

VAR
  rsp32 : LONGINT;

BEGIN

   SproQuery := RNBOsproQuery( ApiPacket,
                               address,
                               queryData,
                               response,
                               @rsp32,
                               length );

  response32 := rsp32;

END; { SproQuery }

FUNCTION SproSetHeartBeat(ApiPacket : RB_SPRO_APIPACKET_PTR;
                          heartbeat : LongInt) : WORD;


BEGIN
        SproSetHeartBeat := RNBOsproSetHeartBeat(ApiPacket,heartbeat);

END;

FUNCTION SproSetProtocol( ApiPacket : RB_SPRO_APIPACKET_PTR;
                         ProtocolFlag : WORD) : WORD;


BEGIN
        SproSetProtocol := RNBOsproSetProtocol(ApiPacket, ProtocolFlag);
END;


END.
